"""
platform.system()
    Linux: Linux
    Mac: Darwin
    Windows: Windows

can change the windows server process name (PyPi modules)
    wmi-client-wrapper (0.0.12)      - Linux-only wrapper around wmi-client for WMI (Windows)
    WMI (1.4.9)                      - Windows Management Instrumentation
    wmi-client-wrapper-py3 (2018.3)  - A fork of Bryan Bishop's wmi-client-wrapper extended to include Python3 support

https://opensource.com/resources/python/gui-frameworks


> wrap/utilize game lobby / config


        ** causes unofficial, custom game
            desired map name
            desired opponent playerID
            fog of war
            realtime or step rate



    UI to select options
        select game critera
            desiredRace
            agent to play
            generated comm

"""


import platform

